-module(problem).
-include("../include/tsp.hrl").
-export([load_from_file/1]).

load_from_file(Filename) ->
	{ok, File} = file:open(Filename, [read]),
	NumCities = line_to_integer(get_line(File)),
	CityNames = get_city_names(File, NumCities),
	Distances = get_distances(File, NumCities),
	#problem{num_cities = NumCities, cities = CityNames, distances = Distances}.

get_city_names(File, N) ->
	get_city_names(File, N, 0, []).
get_city_names(_File, 0, _Id, Names) ->
	lists:reverse(Names);
get_city_names(File, N, Id, Names) ->
	Name = remove_linebreak(get_line(File)),
	get_city_names(File, N-1, Id+1, [{Id, Name}|Names]).

get_distances(File, Upto) ->
	get_distances(File, 0, Upto, dict:new()).
% Loop from Current = 0 to Current = Upto
get_distances(_File, N, N, Dists) ->
	Dists;
get_distances(File, Current, Upto, D1) ->
	D2 = get_distances2(File, Current, Current+1, Upto, D1),
	get_distances(File, Current+1, Upto, D2).


%% "Inner" loop from To to Upto
get_distances2(_File, _From, N, N, Dists) ->
	Dists;
get_distances2(File, From, To, Upto, D1) ->
	Distance = line_to_integer(get_line(File)),
	D2 = dict:store({From, To}, Distance, D1),
	get_distances2(File, From, To+1, Upto, D2).

line_to_integer(String) ->
	list_to_integer(remove_linebreak(String)).

remove_linebreak(String) ->
	[_|Rev] = lists:reverse(String),
	lists:reverse(Rev).

get_line(File) ->
	io:get_line(File, "").
